SERENDALE_QUEST_CONTRACT_ADDRESS_V1 = '0x3132c76acF2217646fB8391918D28a16bD8A8Ef4'
SERENDALE_QUEST_CONTRACT_ADDRESS_V2 = '0xB465F4590095daD50FEe6Ee0B7c6700AC2b04dF8'
CRYSTALVALE_QUEST_CONTRACT_ADDRESS = '0xAd51199B453075C73FA106aFcAAD59f705EF7872'
