SERENDALE_QUEST_CONTRACT_ADDRESS_V1 = '0xE259e8386d38467f0E7fFEdB69c3c9C935dfaeFc'
SERENDALE_QUEST_CONTRACT_ADDRESS_V2 = '0xADFFD2A255B3792873a986895c6312e8FbacFc8B'
CRYSTALVALE_QUEST_CONTRACT_ADDRESS = '0x407ab39B3675f29A719476af6eb3B9E5d93969E6'
